@@ -1,39 +1,13 @@
 from __future__ import annotations
 
-from abc import abstractmethod, abstractproperty
+import calendar
 from dataclasses import dataclass
-from datetime import datetime, timedelta
-from typing import overload, Optional, Set, Protocol
-
-
-class TimeRange(Protocol):
-
-    @abstractmethod
-    def __contains__(self, moment: datetime) -> bool:
-        """
-        Check whether a moment is withing the TimeRange. Both endpoints are included.
-        """
-
-    @property
-    @abstractmethod
-    def to_timedelta(self) -> timedelta:
-        pass
-
-    # @abstractmethod
-    # def __and__(self, other: TimeRange) -> Optional[TimeRange]:
-    #     """
-    #     Find the intersection of the two TimeRange's: A ⋂ B.
-    #     """
-
-    @abstractmethod
-    def __add__(self, other: TimeRange) -> Optional[TimeRange]:
-        """
-        Find the union of the two TimeRange's: A ⋃ B.
-        """
+from datetime import datetime, date, timedelta, time
+from typing import overload, Optional, Set
 
 
 @dataclass(frozen=True)
-class ContinuousTimeRange(TimeRange):
+class ContinuousTimeRange:
     start: datetime
     end: datetime
 
@@ -45,7 +19,7 @@ class ContinuousTimeRange(TimeRange):
         return self.start <= moment <= self.end
 
     @property
-    def to_timedelta(self) -> timedelta:
+    def length(self) -> timedelta:
         return self.end - self.start
 
     def __and__(self, other: ContinuousTimeRange) -> Optional[ContinuousTimeRange]:
@@ -62,13 +36,23 @@ class ContinuousTimeRange(TimeRange):
             return None
         return ContinuousTimeRange(min(self.start, other.start), max(self.end, other.end))
 
+    def __ge__(self, other) -> bool:  # TODO TEST
+        """
+        Check if self (S) is a superset of other (O): S ⊇ O
+        """
+        return self.start <= other.start and other.end <= self.end
+
+    def __le__(self, other) -> bool:  # TODO TEST
+        """
+        Check if self (S) is a subset of other (O): S ⊆ O
+        """
+        return other.start <= self.start and self.end <= other.end
+
     def intersects_with(self, other: ContinuousTimeRange) -> bool:
-        # FIXME This does not work if `other` is a superset of `this`
-        #  e.g. self=today, other=this_year
-        return other.start in self or other.end in self
+        return other.start in self or other.end in self or self <= other
 
 
-class DiscontinuousTimeRange(TimeRange):
+class TimeRange:
 
     @overload
     def __init__(self):
@@ -92,6 +76,9 @@ class DiscontinuousTimeRange(TimeRange):
         if start:
             self._periods.add(ContinuousTimeRange(start, end or start + duration))
 
+    def __eq__(self, other: TimeRange) -> bool:
+        return self._periods == other._periods
+
     def __repr__(self) -> str:
         return ' + '.join(
             [f"TimeRange(start={repr(p.start)}, end={repr(p.end)})" for p in self._periods]
@@ -106,7 +93,10 @@ class DiscontinuousTimeRange(TimeRange):
     def __contains__(self, moment: datetime) -> bool:
         return any([moment in p for p in self._periods])
 
-    def __add__(self, other) -> DiscontinuousTimeRange:
+    def __add__(self, other) -> TimeRange:
+        """
+        Find the union of the two TimeRange's: A ⋃ B.
+        """
         if type(self) != type(other):
             return NotImplemented
         intersectionless_periods: Set[ContinuousTimeRange] = set()
@@ -118,10 +108,67 @@ class DiscontinuousTimeRange(TimeRange):
                     break
             else:
                 intersectionless_periods.add(p)
-        result = DiscontinuousTimeRange()
+        result = TimeRange()
         result._periods = intersectionless_periods
         return result
 
+    # def __and__(self, other: TimeRange) -> Optional[TimeRange]:
+    #     """
+    #     Find the intersection of the two TimeRange's: A ⋂ B.
+    #     """
+    #     raise NotImplementedError()
+
     @property
     def to_timedelta(self) -> timedelta:
-        return sum([p.to_timedelta for p in self._periods], start=timedelta())
+        return sum([p.length for p in self._periods], start=timedelta())
+
+    @property
+    def start(self) -> datetime:
+        """
+        Return the earliest moment in the TimeRange.
+        """
+        return min({p.start for p in self._periods})
+
+    @property
+    def end(self) -> datetime:
+        """
+        Return the latest moment in the TimeRange.
+        """
+        return max({p.end for p in self._periods})
+
+
+class CalendarMonth(TimeRange):
+    """
+    Represent a calendar month.
+    """
+
+    def __init__(self, year: int, month: int):
+        start_date = date(year, month, 1)
+        end_date = self._last_date_of_month(year, month)
+        super().__init__(self._to_datetime(start_date), self._to_datetime(end_date))
+        self.year = year
+        self.month = month
+
+    def __repr__(self):
+        return f'CalendarMonth(year={self.year}, month={self.month})'
+
+    @property
+    def next(self) -> CalendarMonth:
+        """Return an instance of next month."""
+        first_day_in_next_month = self.end.date() + timedelta(days=1)
+        return CalendarMonth(first_day_in_next_month.year, first_day_in_next_month.month)
+
+    @property
+    def prev(self) -> CalendarMonth:
+        """Return an instance of previous month."""
+        last_day_in_previous_month = self.start.date() - timedelta(days=1)
+        return CalendarMonth(last_day_in_previous_month.year, last_day_in_previous_month.month)
+
+    @staticmethod
+    def _last_date_of_month(year: int, month: int) -> datetime.date:
+        _, last_day = calendar.monthrange(year, month)
+        return date(year, month, last_day)
+
+    @staticmethod
+    def _to_datetime(d: date) -> datetime:
+        return datetime.combine(d, time.min)
